<?php
/*
* @package		PayKeeper payment plugin for j2store

* @subpackage	J2Store
* @author    	Michael Kochkin
* @copyright	Copyright (c) 2019 PayKeeper company. All rights reserved.
* --------------------------------------------------------------------------------
*/

/** ensure this file is being included by a parent file */
defined('_JEXEC') or die('Restricted access');
require_once (JPATH_ADMINISTRATOR.'/components/com_j2store/library/plugins/payment.php');
require_once('paykeeper_common_class/paykeeper.class.php');
class plgJ2StorePayment_paykeeper extends J2StorePaymentPlugin
{
	/**
	 * @var $_element  string  Should always correspond with the plugin's filename,
	 *                         forcing it to be unique
	 */
	var $_element    = 'payment_paykeeper';
	
    private $fiscal_cart = array(); //fz54 cart
    private $order_total = 0; //order total sum
    private $shipping_price = 0; //shipping price
    private $use_taxes = false;
    private $use_delivery = false;
    private $order_params = NULL;
    private $discounts = array();

	/**
	 * Constructor
	 *
	 * @param object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
	 * @since 1.5
	 */
	function __construct(& $subject, $config) {
		parent::__construct($subject, $config);
		$this->loadLanguage( '', JPATH_ADMINISTRATOR );	
	}
	
	
	/**
	 * Prepares variables for the payment form. 
	 * Displayed when customer selects the method in Shipping and Payment step of Checkout
	 *
	 * @return unknown_type
	 */
	function _renderForm( $data )
	{
		$vars = new JObject();
		$vars->message = 'Оплата картами Visa и MasterCard на сайте.';
		$html = $this->_getLayout('message', $vars);

		//get the thank you message from the article (ID) provided in the plugin params
		$html .= $this->_displayArticle();

		return $html;
	}

	/**
	 * Method to display a Place order button either to redirect the customer or process the credit card information.
	 * @param $data     array       form post data
	 * @return string   HTML to display
	 */
	function _prePayment( $data )
	{
		// get component params
		$params = J2Store::config();
		$currency = J2Store::currency();

		// prepare the payment form
		F0FTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_j2store/tables');
		$this->order = F0FTable::getInstance('Order', 'J2StoreTable')->getClone();
		$this->order->load(array('order_id'=>$data['order_id']));
        $orderinfo = $this->order->getOrderInformation();
        $app = JFactory::getApplication();
        
        $pk_obj = new PaykeeperPayment();
        
        //set order parameters
        $pk_obj->setOrderParams($this->order->order_total,    //sum
        $orderinfo->billing_first_name
        . " " .
        $orderinfo->billing_last_name,                  //clientid
        $this->order->order_id,                         //orderid
        $this->order->user_email,                       //client_email
        "",                                             //client_phone
        "",                                             //service_name
        $this->params->get('server', ''),                                //payment form url
        $this->params->get('secret', '')                                 //secret key
    );
    /*
        $this->setOrderParams($this->order->order_total,    //sum
            $orderinfo->billing_first_name
            . " " .
            $orderinfo->billing_last_name,                  //clientid
            $this->order->order_id,                         //orderid
            $this->order->user_email,                       //client_email
            "",                                             //client_phone
            "",                                             //service_name
            $this->params->get('server', ''),                                //payment form url
            $this->params->get('secret', '')                                 //secret key
        );
*/
        //GENERATE FZ54 CART
        
		$cart_data = $this->order->get_line_items();
        $item_index = 0;
        foreach ($cart_data as $cart_item) {
            $taxes = array("tax" => "none", "tax_sum" => 0);
            $name = $cart_item["name"];
            $quantity = $cart_item["quantity"];
            $price = $cart_item["amount"];
            $sum = number_format($price*$quantity, 2, ".", "");
            if ($quantity == 1 && $pk_obj->single_item_index < 0)
            $pk_obj->single_item_index = $item_index;
            if ($quantity > 1 && $pk_obj->more_then_one_item_index < 0)
            $pk_obj->more_then_one_item_index = $item_index;    
            if ($cart_item["tax"] != "none") {
                $tax_rate = $this->getTaxRate($cart_item["tax"], $price, FALSE);
                //$tax_rate = 0;
                $taxes = $pk_obj->setTaxes($tax_rate);
            }
            $pk_obj->updateFiscalCart($pk_obj->getPaymentFormType(), 
                                    $name, $price, $quantity, $sum, $taxes["tax"]);
            $item_index++;
             
        }
              
        //add shipping parameters to cart
        $shipping_tax_rate = 0;
        $shipping_taxes = array("tax" => "none", "tax_sum" => 0);
        $shipping_code = $app->input->getString("shipping_code");
        $shipping_price = (float) $app->input->getString("shipping_price");
        $shipping_extra = (float) $app->input->getString("shipping_extra");
        if ($shipping_code != ""){
            array_pop($pk_obj->fiscal_cart);
        }
        if ($app->input->getString("shipping_tax") != "none") {
            $shipping_tax = (float) $app->input->getString("shipping_tax");
            $pk_obj->setShippingPrice($shipping_price+$shipping_extra+$shipping_tax);
            $shipping_tax_rate = $this->getTaxRate($app->input->getString("shipping_tax"),($shipping_price+$shipping_extra), TRUE);
            $shipping_taxes = $pk_obj->setTaxes($shipping_tax_rate);
        }
        else
        $pk_obj->setShippingPrice($shipping_price+$shipping_extra);
        $shipping_name = $app->input->getString("shipping_name");
        if ($pk_obj->getShippingPrice() > 0) {
            $pk_obj->setUseDelivery(); //for precision correct check
            $pk_obj->updateFiscalCart($pk_obj->getPaymentFormType(),
                                    $shipping_name,
                                    $pk_obj->getShippingPrice(), 1, 
                                    $pk_obj->getShippingPrice(), 
                                    $shipping_taxes["tax"],
                                    $shipping_taxes["tax_sum"]);
        }
        
        //set discounts
        $force_discounts_check = $this->params->get('force_discounts_check', '');
        $pk_obj->setDiscounts($force_discounts_check == 1);

        //handle possible precision problem
        $pk_obj->correctPrecision();

        $fiscal_cart_encoded = json_encode($pk_obj->getFiscalCart());

        //generate payment form
        $form = "";
        if ($pk_obj->getPaymentFormType() == "create") { //create form
            $to_hash = $pk_obj->getOrderTotal(TRUE)                .
                       $pk_obj->getOrderParams("clientid")     .
                       $pk_obj->getOrderParams("orderid")      .
                       $pk_obj->getOrderParams("service_name") .
                       $pk_obj->getOrderParams("client_email") .
                       $pk_obj->getOrderParams("client_phone") .
                       $pk_obj->getOrderParams("secret_key");
            $sign = hash ('sha256' , $to_hash);
            $form = '
                <h3>Нажмите на кнопку "Оплатить" для перенаправления на страницу банка.</h3> 
                <form name="payment" id="pay_form" action="'.$pk_obj->getOrderParams("form_url").'" accept-charset="utf-8" method="post">
                <input type="hidden" name="sum" value = "'.$pk_obj->getOrderTotal(TRUE).'"/>
                <input type="hidden" name="orderid" value = "'.$pk_obj->getOrderParams("orderid").'"/>
                <input type="hidden" name="clientid" value = "'.$pk_obj->getOrderParams("clientid").'"/>
                <input type="hidden" name="client_email" value = "'.$pk_obj->getOrderParams("client_email").'"/>
                <input type="hidden" name="client_phone" value = "'.$pk_obj->getOrderParams("client_phone").'"/>
                <input type="hidden" name="service_name" value = "'.$pk_obj->getOrderParams("service_name").'"/>
                <input type="hidden" name="cart" value = \''.htmlentities($fiscal_cart_encoded,ENT_QUOTES).'\' />
                <input type="hidden" name="sign" value = "'.$sign.'"/>
                <input type="submit" class="button btn btn-primary" value="Оплатить"/> 
                </form>';
        }
//        else { //order form
//            $payment_parameters = array(
//                "clientid"=>$this->getOrderParams("clientid"), 
//                "orderid"=>$this->getOrderParams('orderid'), 
//                "sum"=>$this->getOrderTotal(TRUE), 
//                "phone"=>$this->getOrderParams("phone"), 
//                "client_email"=>$this->getOrderParams("client_email"), 
//                "cart"=>$fiscal_cart_encoded);
//            $query = http_build_query($payment_parameters);
//            $options = array("http"=>array(
//                "method"=>"POST",
//                "header"=>"Content-type: application/x-www-form-urlencoded",
//                "content"=>$payment_parameters
//            )); 
//            $context = stream_context_create($options);
//            $err_num = $err_text = NULL;
//            if( function_exists( "curl_init" )) { //using curl
//                $CR = curl_init();
//                curl_setopt($CR, CURLOPT_URL, $this->getOrderParams("form_url"));
//                curl_setopt($CR, CURLOPT_POST, 1);
//                curl_setopt($CR, CURLOPT_FAILONERROR, TRUE); 
//                curl_setopt($CR, CURLOPT_POSTFIELDS, $query);
//                curl_setopt($CR, CURLOPT_RETURNTRANSFER, 1);
//                curl_setopt($CR, CURLOPT_SSL_VERIFYPEER, 0);
//                $result = curl_exec( $CR );
//                $error = curl_error( $CR );
//                return $result;
//                exit;
//                if( !empty( $error )) {
//                    $form = "<br/><span class=message>"."INTERNAL ERROR:".$error."</span>";
//                    return false;
//                }
//                else {
//                    $form = $result;
//                }
//                curl_close($CR);
//            }
//            else { //using file_get_contents
//                if (!ini_get('allow_url_fopen')) {
//                    $form = "<br/><span class=message>"."INTERNAL ERROR: Option allow_url_fopen is not set in php.ini"."</span>";
//                }
//                else {
//                    $form = file_get_contents($this->getOrderParams("form_url"), false, $context);
//                }
//            }
//        }
        if ($form  == "") {
            $form = '<h3>Произошла ошибка при инциализации платежа</h3><p>$err_num: '.htmlspecialchars($err_text).'</p>';
        }

		$vars = new JObject();
        $vars->form = $form;
		$html = $this->_getLayout('prepayment', $vars);
        return $html;
	}

	/**
	 * Processes the payment form
	 * and returns HTML to be displayed to the user
	 * generally with a success/failed message
	 *
	 * @param $data     array       form post data
	 * @return string   HTML to display
	 */
	function _postPayment( $data )
	{
		// Process the payment
		$app = JFactory::getApplication();
		$paction = $app ->input->getString('paction');

		$vars = new JObject();

		switch ($paction)
		{
			case "callback":
				//process the response from the gateway
				$this->processSale();
				$app->close();
				break;
		}

		return $html;
	}

	
	/**
	 * Processes the sale payment
	 *	 
	 */
	private function processSale() {
		
		$app = JFactory::getApplication();
		$data = $app->input->getArray($_POST);

        //debug
        //$this->showDebugInfo($data);
		
		//load the orderpayment record and set some values
		$order = F0FTable::getInstance ( 'Order', 'J2StoreTable' )->getClone ();
			
        if(isset($data['id']) && isset($data['orderid'])) {
            $secret = $this->params->get('secret', '');
            $id = $data['id'];
            $sum = $data['sum'];
            $clientid = $data['clientid'];
            $orderid = $data['orderid'];
            $key = $data['key'];

            if ($key != md5($id.number_format($sum, 2, '.', '').$clientid.$orderid.$secret))
            {
                echo "Error! Hash mismatch";
                exit;
            }
            else {
                if ($order->load ( array ('order_id' => $orderid) )) {

                    if (abs($sum - $order->order_total) >= 0.005) {
                        echo "Error: incorrect sum!";
                        exit;
                    }
                    $order->add_history(JText::_('J2STORE_CALLBACK_RESPONSE_RECEIVED'));
                    
                    //if payment successful, call : $order->payment_complete ();

                    $order->update_status($this->params->get('payment_status', 4));

                    //save the data
                    if (! $order->store ()) {
                        $errors [] = $order->getError ();
                    }
                    //clear cart
                    $order->empty_cart();
                    //return count ( $errors ) ? implode ( "\n", $errors ) : '';

                    echo "OK ".md5($id.$secret);
                }
                else {
                    echo "Error! Incorrect order id!";
                    exit;
                }
            }
        }
        else echo "Error: empty orderid or id!";
	}

    protected function setOrderParams($order_total = 0, $clientid="", $orderid="", $client_email="", 
        $client_phone="", $service_name="", $form_url="", $secret_key="")
    {
       $this->setOrderTotal($order_total);
       $this->order_params = array(
           "sum" => $order_total,
           "clientid" => $clientid,
           "orderid" => $orderid,
           "client_email" => $client_email,
           "client_phone" => $client_phone,
           "phone" => $client_phone,
           "service_name" => $service_name,
           "form_url" => $form_url,
           "secret_key" => $secret_key,
       );
    }

    protected function getOrderParams($value)
    {
        return array_key_exists($value, $this->order_params) ? $this->order_params["$value"] : False;
    }

    protected function updateFiscalCart($ftype, $name="", $price=0, $quantity=0, $sum=0, $tax="none", $tax_sum=0)
    {
        //update fz54 cart
        if ($ftype === "create") {
            $name = str_replace("\n ", "", $name);
            $name = str_replace("\r ", "", $name);
        }
        $this->fiscal_cart[] = array(
            "name" => $name,
            "price" => $price,
            "quantity" => $quantity,
            "sum" => $sum,
            "tax" => $tax,
            "tax_sum" => number_format($tax_sum, 2, ".", "")
        );
    }

    protected function getFiscalCart()
    {
        return $this->fiscal_cart;
    }

    protected function setDiscounts($cart_sum)
    {
        $cart_sum_with_discount = $cart_sum;

        //check if discount is active
        if ((float) $this->order->order_discount > 0) {
            $discount_modifier_value = ($this->getOrderTotal() - $this->getShippingPrice())/$cart_sum;

            if ($discount_modifier_value < 1) {
                $cart_sum_with_discount = 0;
                $fiscal_cart_count = ($this->getShippingPrice() != 0) ? count($this->getFiscalCart())-1
                    : count($this->getFiscalCart());
                //iterate fiscal cart without shipping
                for ($pos=0; $pos<$fiscal_cart_count; $pos++) {
                    $this->fiscal_cart[$pos]["sum"] *= $discount_modifier_value;
                    $cart_sum_with_discount += $this->fiscal_cart[$pos]["sum"];
                    $this->fiscal_cart[$pos]["price"] = $this->fiscal_cart[$pos]["sum"]/$this->fiscal_cart[$pos]["quantity"];
                    //formatting
                    $this->fiscal_cart[$pos]["price"] = number_format($this->fiscal_cart[$pos]["price"], 3, ".", "");
                    $this->fiscal_cart[$pos]["sum"] = number_format($this->fiscal_cart[$pos]["sum"], 2, ".", "");
                    //recalculate taxes
                    $this->recalculateTaxes($pos);
                }
            }
        }
        return $cart_sum_with_discount;
    }

    protected function correctPrecision($cart_sum)
    {
        //handle possible precision problem
        $fiscal_cart_sum = $cart_sum;
        $total_sum = $this->getOrderTotal(TRUE);
        //add shipping sum to cart sum
        $fiscal_cart_sum += $this->getShippingPrice();
        //if ($this->getShippingPrice() > 0)
        //    $fiscal_cart_sum += $this->fiscal_cart[count($this->fiscal_cart)-1]['sum'];

        //debug_info
        //echo "total: " . $total_sum . " - cart: " . $cart_sum;
        $diff_sum = $fiscal_cart_sum - $total_sum;
        if (abs($diff_sum) <= 0.01) {
            $this->setOrderTotal(number_format($total_sum+$diff_sum, 2, ".", ""));
        }
        else {
            if ($this->getUseDelivery() && ($fiscal_cart_sum < $total_sum)) {
                $delivery_pos = count($this->getFiscalCart())-1;
                $this->fiscal_cart[$delivery_pos]["price"] = number_format(
                                   $this->fiscal_cart[$delivery_pos]["price"]+abs($diff_sum), 2, ".", "");
                $this->fiscal_cart[$delivery_pos]["sum"] = number_format(
                                   $this->fiscal_cart[$delivery_pos]["sum"]+abs($diff_sum), 2, ".", "");
                $this->recalculateTaxes($delivery_pos);
            }
        }
    }

    protected function setOrderTotal($value)
    {
        $this->order_total = $value;
    }

    protected function getOrderTotal($format)
    {
        return ($format) ? number_format($this->order_total, 2, ".", "")
            : $this->order_total;
    }

    protected function setShippingPrice($value)
    {
        $this->shipping_price = $value;
    }

    protected function getShippingPrice()
    {
        return $this->shipping_price;
    }

    protected function getPaymentFormType()
    {
        if (strpos($this->order_params["form_url"], "/order/inline") == TRUE)
            return "order";
        else
            return "create";
    }

    protected function setUseTaxes()
    {
        $this->use_taxes = TRUE;
    }

    protected function getUseTaxes()
    {
        return $this->use_taxes;
    }

    protected function setUseDelivery()
    {
        $this->use_delivery = TRUE;
    }

    protected function getUseDelivery()
    {
        return $this->use_delivery;
    }

    protected function recalculateTaxes($item_pos)
    {
        //recalculate taxes
        switch($this->fiscal_cart[$item_pos]['tax']) {
            case "vat10":
                $this->fiscal_cart[$item_pos]['tax_sum'] = round((float)
                    (($this->fiscal_cart[$item_pos]['sum']/110)*10), 2);
                break;
            case "vat18":
                $this->fiscal_cart[$item_pos]['tax_sum'] = round((float)
                    (($this->fiscal_cart[$item_pos]['sum']/118)*18), 2);
                break;
            case "vat20":
                $this->fiscal_cart[$item_pos]['tax_sum'] = round((float)
                    (($this->fiscal_cart[$item_pos]['sum']/120)*20), 2);
                break;
        }
    }

    protected function setTaxes($sum, $tax_rate)
    {
        $taxes = array("tax" => "none", "tax_sum" => 0);
        switch(number_format($tax_rate, 0, ".", "")) {
            case 10:
                $taxes["tax"] = "vat10";
                $taxes["tax_sum"] = round((float)(($sum/110)*10), 2);
                break;
            case 18:
                $taxes["tax"] = "vat18";
                $taxes["tax_sum"] = round((float)(($sum/118)*18), 2);
                break;
            case 20:
                $taxes["tax"] = "vat20";
                $taxes["tax_sum"] = round((float)(($sum/120)*20), 2);
                break;
        }
        return $taxes;
    }

    protected function getTaxRate($tax, $price, $is_shipping)
    {
        $tax = (float) $tax;
        $price = (float) $price;

        if ($is_shipping)
            return round(($tax/($price/100)), 0);
        else
            return ($this->order->is_including_tax) ? round(($tax*100/($price-$tax)), 0)
                : round(($tax/(($price - $tax)/100)), 0);
    }

    protected function showDebugInfo($obj_to_debug)
    {
        echo "<pre>";
        var_dump($obj_to_debug);
        echo "</pre>";
    }
}
